//
//  BRAddressPickerView.m
//  BRPickerViewDemo
//
//  Created by renbo on 2017/8/11.
//  Copyright © 2017 irenb. All rights reserved.
//
//  最新代码下载地址：https://github.com/91renb/BRPickerView

#import "BRAddressPickerView.h"
#import "NSBundle+BRPickerView.h"

@interface BRAddressPickerView ()<UIPickerViewDataSource, UIPickerViewDelegate>
// 地址选择器
@property (nonatomic, strong) UIPickerView *pickerView;

// 选中的省
@property(nonatomic, strong) BRAddressModel *selectProvinceModel;
// 选中的市
@property(nonatomic, strong) BRAddressModel *selectCityModel;
// 选中的区
@property(nonatomic, strong) BRAddressModel *selectAreaModel;
// 记录省选中的位置
@property(nonatomic, assign) NSInteger provinceIndex;
// 记录市选中的位置
@property(nonatomic, assign) NSInteger cityIndex;
// 记录区选中的位置
@property(nonatomic, assign) NSInteger areaIndex;

@property (nonatomic, copy) NSArray <NSString *>* mSelectValues;

@end

@implementation BRAddressPickerView

#pragma mark - 1.显示地址选择器
+ (void)showAddressPickerWithSelectIndexs:(NSArray <NSNumber *>*)selectIndexs
                              resultBlock:(BRAddressResultBlock)resultBlock {
    [self showAddressPickerWithMode:BRAddressPickerModeArea dataSource:nil selectIndexs:selectIndexs isAutoSelect:NO resultBlock:resultBlock];
}

#pragma mark - 2.显示地址选择器
+ (void)showAddressPickerWithMode:(BRAddressPickerMode)mode
                     selectIndexs:(NSArray <NSNumber *>*)selectIndexs
                     isAutoSelect:(BOOL)isAutoSelect
                      resultBlock:(BRAddressResultBlock)resultBlock {
    [self showAddressPickerWithMode:mode dataSource:nil selectIndexs:selectIndexs isAutoSelect:isAutoSelect resultBlock:resultBlock];
}


#pragma mark - 3.显示地址选择器
+ (void)showAddressPickerWithMode:(BRAddressPickerMode)mode
                       dataSource:(NSArray *)dataSource
                     selectIndexs:(NSArray <NSNumber *>*)selectIndexs
                     isAutoSelect:(BOOL)isAutoSelect
                      resultBlock:(BRAddressResultBlock)resultBlock {
    // 创建地址选择器
    BRAddressPickerView *addressPickerView = [[BRAddressPickerView alloc] initWithPickerMode:mode];
    addressPickerView.dataSourceArr = dataSource;
    addressPickerView.selectIndexs = selectIndexs;
    addressPickerView.isAutoSelect = isAutoSelect;
    addressPickerView.resultBlock = resultBlock;
    // 显示
    [addressPickerView show];
}

#pragma mark - 初始化地址选择器
- (instancetype)initWithPickerMode:(BRAddressPickerMode)pickerMode {
    if (self = [super init]) {
        self.pickerMode = pickerMode;
    }
    return self;
}

#pragma mark - 处理选择器数据
- (void)handlerPickerData {

    // 如果外部没有传入地区数据源，就使用本地的数据源
    NSArray *dataSource = [self br_addressJsonArray];
    
    if (!dataSource || dataSource.count == 0) {
        return;
    }
    self.dataSourceArr = dataSource;
    [self pickerView:self.pickerView didSelectRow:0 inComponent:0];
    // 设置默认值
//    [self handlerDefaultSelectValue];
}

#pragma mark - 获取城市JSON数据
- (NSArray *)br_addressJsonArray {
    static NSArray *cityArray = nil;
    if (!cityArray) {
        // 获取 BRAddressPickerView.bundle
        NSBundle *containnerBundle = [NSBundle bundleForClass:[BRAddressPickerView class]];
        NSString *bundlePath = [containnerBundle pathForResource:@"BRAddressPickerView" ofType:@"bundle"];
        NSBundle *addressPickerBundle = [NSBundle bundleWithPath:bundlePath];
        
        // 获取bundle中的JSON文件
        NSString *filePath = [addressPickerBundle pathForResource:@"BRCity" ofType:@"json"];
//        NSData *data = [NSData dataWithContentsOfFile:filePath];
//        cityArray = [NSJSONSerialization JSONObjectWithData:data options:NSJSONReadingAllowFragments error:nil];
        NSString *citysStr = [NSString stringWithContentsOfFile:filePath encoding:NSUTF8StringEncoding error:nil];
        
        cityArray = [BRAddressModel mj_objectArrayWithKeyValuesArray:[citysStr mj_JSONObject]];
    }
    return cityArray;
}

#pragma mark - 设置默认选择的值
- (void)handlerDefaultSelectValue {
//    __block NSString *selectProvinceName = nil;
//    __block NSString *selectCityName = nil;
//    __block NSString *selectAreaName = nil;
//
//    if (self.mSelectValues.count > 0) {
//        selectProvinceName = self.mSelectValues.count > 0 ? self.mSelectValues[0] : nil;
//        selectCityName = self.mSelectValues.count > 1 ? self.mSelectValues[1] : nil;
//        selectAreaName = self.mSelectValues.count > 2 ? self.mSelectValues[2] : nil;
//    }
//
//    __weak typeof(self) weakSelf = self;
//
//    if (self.pickerMode == BRAddressPickerModeProvince || self.pickerMode == BRAddressPickerModeCity || self.pickerMode == BRAddressPickerModeArea) {
//        if (self.selectIndexs.count > 0) {
//            NSInteger provinceIndex = [self.selectIndexs[0] integerValue];
//            self.provinceIndex = (provinceIndex > 0 && provinceIndex < self.provinceModelArr.count) ? provinceIndex : 0;
//            self.selectProvinceModel = self.provinceModelArr.count > self.provinceIndex ? self.provinceModelArr[self.provinceIndex] : nil;
//        } else {
//            self.provinceIndex = 0;
//            self.selectProvinceModel = self.provinceModelArr.count > 0 ? self.provinceModelArr[0] : nil;
//            [self.provinceModelArr enumerateObjectsUsingBlock:^(BRAddressModel *  _Nonnull model, NSUInteger idx, BOOL * _Nonnull stop) {
//                if (selectProvinceName && [model.name isEqualToString:selectProvinceName]) {
//                    weakSelf.provinceIndex = idx;
//                    weakSelf.selectProvinceModel = model;
//                    *stop = YES;
//                }
//            }];
//        }
//    }
//
//    if (self.pickerMode == BRAddressPickerModeCity || self.pickerMode == BRAddressPickerModeArea) {
//        self.cityModelArr = [self getCityModelArray:self.provinceIndex];
//        if (self.selectIndexs.count > 0) {
//            NSInteger cityIndex = self.selectIndexs.count > 1 ? [self.selectIndexs[1] integerValue] : 0;
//            self.cityIndex = (cityIndex > 0 && cityIndex < self.cityModelArr.count) ? cityIndex : 0;
//            self.selectCityModel = self.cityModelArr.count > self.cityIndex ? self.cityModelArr[self.cityIndex] : nil;
//        } else {
//            self.cityIndex = 0;
//            self.selectCityModel = self.cityModelArr.count > 0 ? self.cityModelArr[0] : nil;
//            [self.cityModelArr enumerateObjectsUsingBlock:^(BRCityModel *  _Nonnull model, NSUInteger idx, BOOL * _Nonnull stop) {
//                if (selectCityName && [model.name isEqualToString:selectCityName]) {
//                    weakSelf.cityIndex = idx;
//                    weakSelf.selectCityModel = model;
//                    *stop = YES;
//                }
//            }];
//        }
//    }
//
//    if (self.pickerMode == BRAddressPickerModeArea) {
//        self.areaModelArr = [self getAreaModelArray:self.provinceIndex cityIndex:self.cityIndex];
//        if (self.selectIndexs.count > 0) {
//            NSInteger areaIndex = self.selectIndexs.count > 2 ? [self.selectIndexs[2] integerValue] : 0;
//            self.areaIndex = (areaIndex > 0 && areaIndex < self.areaModelArr.count) ? areaIndex : 0;
//            self.selectAreaModel = self.areaModelArr.count > self.areaIndex ? self.areaModelArr[self.areaIndex] : nil;
//        } else {
//            self.areaIndex = 0;
//            self.selectAreaModel = self.areaModelArr.count > 0 ? self.areaModelArr[0] : nil;
//            [self.areaModelArr enumerateObjectsUsingBlock:^(BRAreaModel *  _Nonnull model, NSUInteger idx, BOOL * _Nonnull stop) {
//                if (selectAreaName && [model.name isEqualToString:selectAreaName]) {
//                    weakSelf.areaIndex = idx;
//                    weakSelf.selectAreaModel = model;
//                    *stop = YES;
//                }
//            }];
//        }
//    }
}


#pragma mark - 地址选择器
- (UIPickerView *)pickerView {
    if (!_pickerView) {
        CGFloat pickerHeaderViewHeight = self.pickerHeaderView ? self.pickerHeaderView.bounds.size.height : 0;
        _pickerView = [[UIPickerView alloc]initWithFrame:CGRectMake(0, self.pickerStyle.titleBarHeight + pickerHeaderViewHeight, self.keyView.bounds.size.width, self.pickerStyle.pickerHeight)];
        _pickerView.backgroundColor = self.pickerStyle.pickerColor;
        _pickerView.autoresizingMask = UIViewAutoresizingFlexibleBottomMargin | UIViewAutoresizingFlexibleRightMargin | UIViewAutoresizingFlexibleWidth;
        _pickerView.dataSource = self;
        _pickerView.delegate = self;
    }
    return _pickerView;
}

#pragma mark - UIPickerViewDataSource
// 1.设置 pickerView 的列数
- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView {
    switch (self.pickerMode) {
        case BRAddressPickerModeProvince:
            return 1;
            break;
        case BRAddressPickerModeCity:
            return 2;
            break;
        case BRAddressPickerModeArea:
            return 3;
            break;
            
        default:
            break;
    }
}

// 2.设置 pickerView 每列的行数
- (NSInteger)pickerView:(UIPickerView *)pickerView numberOfRowsInComponent:(NSInteger)component {
    if (component == 0) {
        // 返回省个数
        return self.dataSourceArr.count;
    }
    if (component == 1) {
        // 返回市个数
        return self.dataSourceArr[self.provinceIndex].addressModel.count;
    }
    if (component == 2) {
        // 返回区个数
        return self.dataSourceArr[self.provinceIndex].addressModel[self.cityIndex].addressModel.count;
    }
    return 0;
}

#pragma mark - UIPickerViewDelegate
// 3.设置 pickerView 的显示内容
- (UIView *)pickerView:(UIPickerView *)pickerView viewForRow:(NSInteger)row forComponent:(NSInteger)component reusingView:(nullable UIView *)view {
    // 1.自定义 row 的内容视图
    UILabel *label = (UILabel *)view;
    if (!label) {
        label = [[UILabel alloc]init];
        label.backgroundColor = [UIColor clearColor];
        label.textAlignment = NSTextAlignmentCenter;
        label.font = self.pickerStyle.pickerTextFont;
        label.textColor = self.pickerStyle.pickerTextColor;
        // 字体自适应属性
        label.adjustsFontSizeToFitWidth = YES;
        // 自适应最小字体缩放比例
        label.minimumScaleFactor = 0.5f;
    }
    if (component == 0) {
        BRAddressModel *model = self.dataSourceArr[row];
        label.text = NSLocalizedString(model.extName, nil);
    } else if (component == 1) {
        BRAddressModel *model = self.dataSourceArr[self.provinceIndex];
        BRAddressModel *model1 = self.dataSourceArr[self.provinceIndex].addressModel[row];
        NSString *name = [model1.extName isEqualToString:@"市辖区"] ? model.extName : model1.extName;
        label.text = NSLocalizedString(name, nil);
    } else if (component == 2) {
        BRAddressModel *model = self.dataSourceArr[self.provinceIndex].addressModel[self.cityIndex].addressModel[row];
        label.text = NSLocalizedString(model.extName, nil);
    }
    // 2.设置选择器中间选中行的样式
    [self.pickerStyle setupPickerSelectRowStyle:pickerView titleForRow:row forComponent:component];
    
    return label;
}

// 4.滚动 pickerView 执行的回调方法
- (void)pickerView:(UIPickerView *)pickerView didSelectRow:(NSInteger)row inComponent:(NSInteger)component {
    if (component == 0) { // 选择省
        // 保存选择的省份的索引
        self.provinceIndex = row;
        self.cityIndex = 0;
        self.areaIndex = 0;
        self.selectProvinceModel = self.dataSourceArr[row];
        switch (self.pickerMode) {
            case BRAddressPickerModeProvince:
            {

            }
                break;
            case BRAddressPickerModeCity:
            {
                self.selectCityModel = self.dataSourceArr[self.provinceIndex].addressModel[0];
                [self.pickerView reloadComponent:1];
                [self.pickerView selectRow:0 inComponent:1 animated:YES];
            }
                break;
            case BRAddressPickerModeArea:
            {
                self.selectCityModel = self.dataSourceArr[self.provinceIndex].addressModel[0];
                self.selectAreaModel = self.dataSourceArr[self.provinceIndex].addressModel[self.cityIndex].addressModel[0];
                [self.pickerView reloadComponent:1];
                [self.pickerView selectRow:0 inComponent:1 animated:YES];
                [self.pickerView reloadComponent:2];
                [self.pickerView selectRow:0 inComponent:2 animated:YES];
            }
                break;
            default:
                break;
        }
    }
    if (component == 1) { // 选择市
        // 保存选择的城市的索引
        self.cityIndex = row;
        self.areaIndex = 0;
        self.selectCityModel = self.dataSourceArr[self.provinceIndex].addressModel[row];
        switch (self.pickerMode) {
            case BRAddressPickerModeCity:
            {

            }
                break;
            case BRAddressPickerModeArea:
            {
                self.selectAreaModel = self.dataSourceArr[self.provinceIndex].addressModel[self.cityIndex].addressModel[0];
                [self.pickerView reloadComponent:2];
                [self.pickerView selectRow:0 inComponent:2 animated:YES];
            }
                break;
            default:
                break;
        }
    }
    if (component == 2) { // 选择区
        // 保存选择的地区的索引
        self.areaIndex = row;
        self.selectAreaModel = self.dataSourceArr[self.provinceIndex].addressModel[self.cityIndex].addressModel[row];
    }
    
    // 滚动选择时执行 changeBlock
    if (self.changeBlock) {
        self.changeBlock(self.selectProvinceModel, self.selectCityModel, self.selectAreaModel);
    }
    
    // 设置自动选择时，滚动选择时就执行 resultBlock
    if (self.isAutoSelect) {
        if (self.resultBlock) {
            self.resultBlock(self.selectProvinceModel, self.selectCityModel, self.selectAreaModel);
        }
    }
}

// 设置行高
- (CGFloat)pickerView:(UIPickerView *)pickerView rowHeightForComponent:(NSInteger)component {
    return self.pickerStyle.rowHeight;
}

#pragma mark - 重写父类方法
- (void)reloadData {
    // 1.处理数据源
    [self handlerPickerData];
    // 2.刷新选择器
    [self.pickerView reloadAllComponents];
    // 3.滚动到选择的地区
    if (self.pickerMode == BRAddressPickerModeProvince) {
        [self.pickerView selectRow:self.provinceIndex inComponent:0 animated:YES];
    } else if (self.pickerMode == BRAddressPickerModeCity) {
        [self.pickerView selectRow:self.provinceIndex inComponent:0 animated:YES];
        [self.pickerView selectRow:self.cityIndex inComponent:1 animated:YES];
    } else if (self.pickerMode == BRAddressPickerModeArea) {
        [self.pickerView selectRow:self.provinceIndex inComponent:0 animated:YES];
        [self.pickerView selectRow:self.cityIndex inComponent:1 animated:YES];
        [self.pickerView selectRow:self.areaIndex inComponent:2 animated:YES];
    }
}

- (void)addPickerToView:(UIView *)view {
    // 1.添加地址选择器
    if (view) {
        // 立即刷新容器视图 view 的布局（防止 view 使用自动布局时，选择器视图无法正常显示）
        [view setNeedsLayout];
        [view layoutIfNeeded];
        
        self.frame = view.bounds;
        CGFloat pickerHeaderViewHeight = self.pickerHeaderView ? self.pickerHeaderView.bounds.size.height : 0;
        CGFloat pickerFooterViewHeight = self.pickerFooterView ? self.pickerFooterView.bounds.size.height : 0;
        self.pickerView.frame = CGRectMake(0, pickerHeaderViewHeight, view.bounds.size.width, view.bounds.size.height - pickerHeaderViewHeight - pickerFooterViewHeight);
        [self addSubview:self.pickerView];
    } else {
        // iOS16：重新设置 pickerView 高度（解决懒加载设置frame不生效问题）
        CGFloat pickerHeaderViewHeight = self.pickerHeaderView ? self.pickerHeaderView.bounds.size.height : 0;
        self.pickerView.frame = CGRectMake(0, self.pickerStyle.titleBarHeight + pickerHeaderViewHeight, self.keyView.bounds.size.width, self.pickerStyle.pickerHeight);
        
        [self.alertView addSubview:self.pickerView];
    }
    
    // ③添加中间选择行的两条分割线
    if (self.pickerStyle.clearPickerNewStyle) {
        [self.pickerStyle addSeparatorLineView:self.pickerView];
    }
    
    // 2.绑定数据
    [self reloadData];
    
    __weak typeof(self) weakSelf = self;
    self.doneBlock = ^{
        // 点击确定按钮后，执行block回调
        if (weakSelf.resultBlock) {
            weakSelf.resultBlock(weakSelf.selectProvinceModel, weakSelf.selectCityModel, weakSelf.selectAreaModel);
        }
    };
    
    [super addPickerToView:view];
}

#pragma mark - 重写父类方法
- (void)addSubViewToPicker:(UIView *)customView {
    [self.pickerView addSubview:customView];
}

#pragma mark - 弹出选择器视图
- (void)show {
    [self addPickerToView:nil];
}

#pragma mark - 关闭选择器视图
- (void)dismiss {
    [self removePickerFromView:nil];
}

#pragma mark - setter方法
- (void)setSelectValues:(NSArray<NSString *> *)selectValues {
    self.mSelectValues = selectValues;
}

#pragma mark - getter方法

- (BRAddressModel *)selectProvinceModel {
    if (!_selectProvinceModel) {
        _selectProvinceModel = [[BRAddressModel alloc]init];
    }
    return _selectProvinceModel;
}

- (BRAddressModel *)selectCityModel {
    if (!_selectCityModel) {
        _selectCityModel = [[BRAddressModel alloc]init];
        _selectCityModel.code = @"";
        _selectCityModel.name = @"";
    }
    return _selectCityModel;
}

- (BRAddressModel *)selectAreaModel {
    if (!_selectAreaModel) {
        _selectAreaModel = [[BRAddressModel alloc]init];
        _selectAreaModel.code = @"";
        _selectAreaModel.name = @"";
    }
    return _selectAreaModel;
}

- (NSArray *)dataSourceArr {
    if (!_dataSourceArr) {
        _dataSourceArr = [NSArray array];
    }
    return _dataSourceArr;
}

- (NSArray<NSString *> *)mSelectValues {
    if (!_mSelectValues) {
        _mSelectValues = [NSArray array];
    }
    return _mSelectValues;
}

- (NSArray<NSNumber *> *)selectIndexs {
    if (!_selectIndexs) {
        _selectIndexs = [NSArray array];
    }
    return _selectIndexs;
}

@end
